@@ -1,5 +1,4 @@
 from bisect import bisect
-from copy import deepcopy
 from typing import (Callable, Dict, Generic, Optional, Sequence, Tuple,
                     TypeVar,
                     Union)
@@ -90,7 +89,6 @@ class WeightedCollection(Generic[ITEM]):
             old_weight = self._weights[i]
             w = weight_func(item, old_weight)
             if w > 0:
-                item = deepcopy(item)
                 result.append((item, w))
         return WeightedCollection[ITEM](result, self._rng)
 
