@@ -5,6 +5,9 @@ __all__ = ['RNG']
 from .primitive import PrimitiveRandomNumberGenerator
 from .lcg import LCG1
 
+LOG4 = math.log(4.0)
+SG_MAGICCONST = 1.0 + math.log(4.5)
+
 
 class RNG(object):
     """
@@ -29,8 +32,8 @@ class RNG(object):
             self._generator = generator_or_seed
         else:
             raise ValueError(
-                "generatorOrSeed must be an int or a "
-                "PrimitiveRandomNumberGenerator")
+                    "generatorOrSeed must be an int or a "
+                    "PrimitiveRandomNumberGenerator")
         self._nextNormal = None
 
     # primitives: boolean, sign, integer, float
@@ -127,7 +130,8 @@ class RNG(object):
         """
         Roll [num_dice] x [die_size]-sided dice and return the sum.
 
-        The dice are what you would expect in the real world: an integer in the
+        The dice are what you would expect in the real world: an integer in
+        the
         range [1,die_size].
         """
         return num_dice + self.i_sum(num_dice, die_size)
@@ -136,7 +140,8 @@ class RNG(object):
         """
         Return sum of two uniform random deviates.  Result will lie in
         [center-half_width, center+half_width) and have a triangular
-        distribution with a standard deviation of approximately 0.41*half_width.
+        distribution with a standard deviation of approximately
+        0.41*half_width.
 
         :param half_width: half width of the resulting distribution
         :type half_width: float
@@ -162,8 +167,8 @@ class RNG(object):
         :rtype: float
         """
         return center + (
-                            self.f() + self.f() + self.f()) * half_width / \
-                        1.5 - \
+                self.f() + self.f() + self.f()) * half_width / \
+               1.5 - \
                half_width
 
     def f4(self, half_width=1.0, center=0.0):
@@ -215,7 +220,8 @@ class RNG(object):
         :rtype: float
         """
         return center + (self.f() + self.f() + self.f() +
-                         self.f() + self.f() + self.f()) * half_width / 3.0 - \
+                         self.f() + self.f() + self.f()) * half_width / 3.0 \
+               - \
                half_width
 
     def f2r(self, a, b):
@@ -471,13 +477,111 @@ class RNG(object):
             return center + x * left_stddev
         return center + x * right_stddev
 
-    def inverse_normal(self,mean:float, shape:float)->float:
+    def inverse_normal(self, mean: float, shape: float) -> float:
         z = self.normal(1)
-        y = z*z
-        x = mean+mean*mean*y/(2*shape)-mean/(2*shape)*math.sqrt(
-            4*mean*shape*y+mean*mean*y*y)
-        v=self.f()
-        if v<=mean/(mean+x):
+        y = z * z
+        x = mean + mean * mean * y / (2 * shape) - mean / (
+                2 * shape) * math.sqrt(
+                4 * mean * shape * y + mean * mean * y * y)
+        v = self.f()
+        if v <= mean / (mean + x):
             return x
-        return mean*mean/x
+        return mean * mean / x
 
+    # gammavariate and betavariate are taken almost verbatim from the
+    # Python Standard Library v3.10
+
+    def gammavariate(self, alpha: float, beta: float) -> float:
+        """Gamma distribution.  Not the gamma function!
+        Conditions on the parameters are alpha > 0 and beta > 0.
+        The probability distribution function is:
+                    x ** (alpha - 1) * math.exp(-x / beta)
+          pdf(x) =  --------------------------------------
+                      math.gamma(alpha) * beta ** alpha
+        """
+        # alpha > 0, beta > 0, mean is alpha*beta, variance is alpha*beta**2
+
+        # Warning: a few older sources define the gamma distribution in terms
+        # of alpha > -1.0
+        if alpha <= 0.0 or beta <= 0.0:
+            raise ValueError('gammavariate: alpha and beta must be > 0.0')
+
+        if alpha > 1.0:
+
+            # Uses R.C.H. Cheng, "The generation of Gamma
+            # variables with non-integral shape parameters",
+            # Applied Statistics, (1977), 26, No. 1, p71-74
+
+            ainv = math.sqrt(2.0 * alpha - 1.0)
+            bbb = alpha - LOG4
+            ccc = alpha + ainv
+
+            while True:
+                u1 = self.f()
+                if not 1e-7 < u1 < 0.9999999:
+                    continue
+                u2 = 1.0 - self.f()
+                v = math.log(u1 / (1.0 - u1)) / ainv
+                x = alpha * math.exp(v)
+                z = u1 * u1 * u2
+                r = bbb + ccc * v - x
+                if r + SG_MAGICCONST - 4.5 * z >= 0.0 or r >= math.log(z):
+                    return x * beta
+
+        elif alpha == 1.0:
+            # expovariate(1/beta)
+            return -math.log(1.0 - self.f()) * beta
+
+        else:
+            # alpha is between 0 and 1 (exclusive)
+            # Uses ALGORITHM GS of Statistical Computing - Kennedy & Gentle
+            while True:
+                u = self.f()
+                b = (math.e + alpha) / math.e
+                p = b * u
+                if p <= 1.0:
+                    x = p ** (1.0 / alpha)
+                else:
+                    x = -math.log((b - p) / alpha)
+                u1 = self.f()
+                if p > 1.0:
+                    if u1 <= x ** (alpha - 1.0):
+                        break
+                elif u1 <= math.exp(-x):
+                    break
+            return x * beta
+
+    def betavariate(self, alpha: float, beta: float) -> float:
+        """Beta distribution.
+        Conditions on the parameters are alpha > 0 and beta > 0.
+        Returned values range between 0 and 1.
+        """
+        ## See
+        ## http://mail.python.org/pipermail/python-bugs-list/2001
+        # -January/003752.html
+        ## for Ivan Frohne's insightful analysis of why the original
+        # implementation:
+        ##
+        ##    def betavariate(self, alpha, beta):
+        ##        # Discrete Event Simulation in C, pp 87-88.
+        ##
+        ##        y = self.expovariate(alpha)
+        ##        z = self.expovariate(1.0/beta)
+        ##        return z/(y+z)
+        ##
+        ## was dead wrong, and how it probably got that way.
+
+        # This version due to Janne Sinkkonen, and matches all the std
+        # texts (e.g., Knuth Vol 2 Ed 3 pg 134 "the beta distribution").
+        y = self.gammavariate(alpha, 1.0)
+        if y:
+            return y / (y + self.gammavariate(beta, 1.0))
+        return 0.0
+
+    def pert(self,
+             a: float, b: float, c: float,
+             gamma: float = 4) -> float:
+        r = c - a
+        alpha = 1 + gamma * (b - a) / r
+        beta = 1 + gamma * (c - b) / r
+        return a + self.betavariate(alpha, beta) * r
